==> erpmc-0.1.0.tar.gz: lib/store_management/product/model/model.dart <==
part of '../../../store_management.dart';

class ProductModel extends NameModel<int> {
  static const String typeIdTag = "typeId";
  final int typeId;
  static const String descriptionTag = "description";
  final String? description;
  static const String secondNameTag = "secondName";
  final String? secondName;
  static const String barcodeTag = "barcode";
  final String? barcode;
  static const String imagesTag = "images";
  final List<String>? images;
  static const String hasExpireDateTag = "hasExpireDate";
  final bool hasExpireDate;
  static const String isServiceTag = "isService";
  final bool isService;

  const ProductModel({
    super.id = 0,
    required super.name,
    bool super.status = true,
    required this.typeId,
    this.description,
    this.barcode,
    this.images,
    this.secondName,
    this.isService = false,
    this.hasExpireDate = false,
  });

  ProductModel.fromMap(super.map)
      : description = map.getString(descriptionTag),
        typeId = map.getInt(typeIdTag)!,
        barcode = map.getString(barcodeTag),
        images = map.getList(imagesTag)?.cast<String>(),
        secondName = map.getString(secondNameTag),
        isService = map.getBool(isServiceTag)!,
        hasExpireDate = map.getBool(hasExpireDateTag)!,
        super.fromMap();

  @override
  Map<String, dynamic> toMap() => {
        ...super.toMap(),
        descriptionTag: description,
        typeIdTag: typeId,
        barcodeTag: barcode,
        imagesTag: images,
        secondNameTag: secondName,
        isServiceTag: isService,
        hasExpireDateTag: hasExpireDate,
      };
}

extension ProductModelExtension<ModelType extends ProductModel>
    on Iterable<ModelType> {
  bool _filter({
    required ModelType element,
    bool? status,
    bool? isService,
    bool? hasExpireDate,
    Set<int>? typeIdSet,
    int? typeId,
  }) {
    var result = true;
    result &= element.status == status;
    if (isService != null) result &= element.isService == isService;
    if (hasExpireDate != null) result &= element.hasExpireDate == hasExpireDate;

    if (typeIdSet != null) {
      result &= typeIdSet.contains(element.typeId);
    } else if (typeId != null) {
      result &= element.typeId == typeId;
    }
    return result;
  }

  ModelType? firstWhereByBarcode(String? barcode) =>
      firstWhereOrNull((element) => element.barcode == barcode);

  ModelType? firstWhereFilter({
    bool? status,
    bool? isService,
    bool? hasExpireDate,
    Set<int>? typeIdSet,
    int? typeId,
    bool not = false,
  }) =>
      firstWhereOrNull((element) =>
          not ^
          _filter(
            element: element,
            status: status,
            isService: isService,
            hasExpireDate: hasExpireDate,
            typeId: typeId,
            typeIdSet: typeIdSet,
          ));
  ModelType? lastWhereFilter({
    bool? status,
    bool? isService,
    bool? hasExpireDate,
    Set<int>? typeIdSet,
    int? typeId,
    bool not = false,
  }) =>
      lastWhereOrNull((element) =>
          not ^
          _filter(
            element: element,
            status: status,
            isService: isService,
            hasExpireDate: hasExpireDate,
            typeId: typeId,
            typeIdSet: typeIdSet,
          ));

  Iterable<ModelType> whereByFilter({
    bool? status,
    bool? isService,
    bool? hasExpireDate,
    Set<int>? typeIdSet,
    int? typeId,
    bool not = false,
  }) =>
      where((element) =>
          not ^
          _filter(
            element: element,
            status: status,
            isService: isService,
            hasExpireDate: hasExpireDate,
            typeId: typeId,
            typeIdSet: typeIdSet,
          ));
}

==> erpmc-0.1.2.tar.gz: lib/store_management/product/model/model.dart <==
part of '../../../store_management.dart';

class ProductModel extends NameModel<int> {
  static const String typeIdTag = "typeId";
  final int typeId;
  static const String descriptionTag = "description";
  final String? description;
  static const String secondNameTag = "secondName";
  final String? secondName;
  static const String barcodeTag = "barcode";
  final String? barcode;
  static const String imagesTag = "images";
  final List<String>? images;
  static const String hasExpireDateTag = "hasExpireDate";
  final bool hasExpireDate;
  static const String isServiceTag = "isService";
  final bool isService;

  const ProductModel({
    super.id = 0,
    required super.name,
    super.status = true,
    required this.typeId,
    this.description,
    this.barcode,
    this.images,
    this.secondName,
    this.isService = false,
    this.hasExpireDate = false,
  });

  ProductModel.fromMap(super.map)
      : description = map.getString(descriptionTag),
        typeId = map.getInt(typeIdTag)!,
        barcode = map.getString(barcodeTag),
        images = map.getList(imagesTag)?.cast<String>(),
        secondName = map.getString(secondNameTag),
        isService = map.getBool(isServiceTag)!,
        hasExpireDate = map.getBool(hasExpireDateTag)!,
        super.fromMap();

  @override
  Map<String, dynamic> toMap() => {
        ...super.toMap(),
        descriptionTag: description,
        typeIdTag: typeId,
        barcodeTag: barcode,
        imagesTag: images,
        secondNameTag: secondName,
        isServiceTag: isService,
        hasExpireDateTag: hasExpireDate,
      };
}

extension ProductModelExtension<ModelType extends ProductModel>
    on Iterable<ModelType> {
  bool _filter({
    required ModelType element,
    bool? status,
    bool? isService,
    bool? hasExpireDate,
    Set<int>? typeIdSet,
    int? typeId,
  }) {
    var result = true;
    result &= element.status == status;
    if (isService != null) result &= element.isService == isService;
    if (hasExpireDate != null) result &= element.hasExpireDate == hasExpireDate;

    if (typeIdSet != null) {
      result &= typeIdSet.contains(element.typeId);
    } else if (typeId != null) {
      result &= element.typeId == typeId;
    }
    return result;
  }

  ModelType? firstWhereByBarcode(String? barcode) =>
      firstWhereOrNull((element) => element.barcode == barcode);

  ModelType? firstWhereFilter({
    bool? status,
    bool? isService,
    bool? hasExpireDate,
    Set<int>? typeIdSet,
    int? typeId,
    bool not = false,
  }) =>
      firstWhereOrNull((element) =>
          not ^
          _filter(
            element: element,
            status: status,
            isService: isService,
            hasExpireDate: hasExpireDate,
            typeId: typeId,
            typeIdSet: typeIdSet,
          ));
  ModelType? lastWhereFilter({
    bool? status,
    bool? isService,
    bool? hasExpireDate,
    Set<int>? typeIdSet,
    int? typeId,
    bool not = false,
  }) =>
      lastWhereOrNull((element) =>
          not ^
          _filter(
            element: element,
            status: status,
            isService: isService,
            hasExpireDate: hasExpireDate,
            typeId: typeId,
            typeIdSet: typeIdSet,
          ));

  Iterable<ModelType> whereByFilter({
    bool? status,
    bool? isService,
    bool? hasExpireDate,
    Set<int>? typeIdSet,
    int? typeId,
    bool not = false,
  }) =>
      where((element) =>
          not ^
          _filter(
            element: element,
            status: status,
            isService: isService,
            hasExpireDate: hasExpireDate,
            typeId: typeId,
            typeIdSet: typeIdSet,
          ));
}
